// Copyright (C) 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using System.Data;
using System.Threading;
using MySql.Data.MySqlClient;
using NUnit.Framework;

namespace MySql.Data.MySqlClient.Tests
{
	/// <summary>
	/// Summary description for PoolingTests.
	/// </summary>
	[TestFixture]
	public class PoolingTests : BaseTest
	{
		[TestFixtureSetUp]
		public void FixtureSetup()
		{
			csAdditions = ";pooling=true";
			Open();
		}

		[TestFixtureTearDown]
		public void FixtureTearDown()
		{
			Close();
		}

		[Test]
		public void Connection() 
		{
			execSQL("SET @@global.wait_timeout=15");
			execSQL("SET @@local.wait_timeout=28800");

			MySqlDataAdapter da = new MySqlDataAdapter("show processlist", conn);
			DataTable dt = new DataTable();
			da.Fill(dt);
			int count = dt.Rows.Count;
			dt.Clear();

			string connStr = conn.ConnectionString;
			// first test that only a single connection get's used
			for (int i=0; i < 10; i++) 
			{
				MySqlConnection c = new MySqlConnection( connStr );
				c.Open();
				c.Close();

				dt.Clear();
				da.Fill(dt);
				Assert.IsTrue( dt.Rows.Count <= (count+1), "During initial pool opening" );
			}

			Thread.Sleep(20000);
			dt.Clear();
			da.Fill(dt);
			Assert.IsTrue( dt.Rows.Count <= count, "After first sleep" );

			connStr += ";Min Pool Size=10";
			MySqlConnection[] connArray = new MySqlConnection[10];
			for (int i=0; i < connArray.Length; i++) 
			{
				connArray[i] = new MySqlConnection( connStr );
				connArray[i].Open();

				dt.Clear();
				da.Fill(dt);
				Assert.IsTrue( dt.Rows.Count <= (count+10), "After opening with Min Pool Size=10" );
			}

			for (int i=0; i < connArray.Length; i++)
				connArray[i].Close();


			Thread.Sleep(20000);
			dt.Clear();
			da.Fill(dt);
			Assert.IsTrue( dt.Rows.Count <= count, "After second sleeping" );

			execSQL( "SET @@global.wait_timeout=28800" );
		}

		[Test]
		public void ReclaimBrokenConnection() 
		{
			execSQL("SET @@global.wait_timeout=15");
			execSQL("SET @@local.wait_timeout=28800");

			// now create a new connection string only allowing 5 connections in the pool
			string connStr = conn.ConnectionString + ";max pool size=1";

			// now use up those 5 connections very fast.
			MySqlConnection c = new MySqlConnection( connStr );
			c.Open();

			// now attempting to open a connection should fail
			try 
			{
				MySqlConnection c2 = new MySqlConnection( connStr );
				c2.Open();
				Assert.Fail( "Open after using up pool should fail" );
			}
			catch (Exception) { }

			// now wait 20 seconds for the connections to die
			Thread.Sleep(20000);

			// Opening a connection now should work
			try 
			{
				MySqlConnection c2 = new MySqlConnection( connStr );
				c2.Open();
				c2.Close();
			}
			catch (Exception ex) 
			{ 
				Assert.Fail( ex.Message );
			}

			// first set our server wait timeout to 15 seconds
			execSQL( "SET @@global.wait_timeout=28800" );
		}

		[Test()]
		public void TestUserReset()
		{
			execSQL("SET @testvar='5'");
			MySqlCommand cmd = new MySqlCommand("SELECT @testvar", conn);
			object var = cmd.ExecuteScalar();
			Assert.AreEqual("5", var );
			conn.Close();

			conn.Open();
			object var2 = cmd.ExecuteScalar();
			Assert.AreEqual( DBNull.Value, var2 );
			conn.Close();
		}

	}
}
